/////////////////////////////////////////////////////////////////////////////
// Project:     wxIView
// Purpose:     Complex wxWidgets sample
// Name:        wxIViewPrintout.h
// Author:      Daniel Nell
// Created:     2005/10/18
// Copyright:   (c) Daniel Nell
// License:     wxWindows license
// Modified by:
/////////////////////////////////////////////////////////////////////////////

#ifndef __WXIVIEWPRINTOUT_H
#define __WXIVIEWPRINTOUT_H

#define wxivIMAGEPPIX (180)
#define wxivIMAGEPPIY (180)


// Forward declartion.
class wxIViewPaintBase;


/////////////////////////////////////////////////////////////////////////////
//  Class wxIViewPrintout
//
/// \brief Class to print out an image.
///
/// Images can be multipage images, too.
class wxIViewPrintout : public wxPrintout
{
  public:
    /// \brief Standard constructor.
    ///
    /// \param poPaint              a pointer to an instance of wxIViewPaintBase
    /// \param strTitle             the window caption text, defaults to "wxIViewPrintout"
    /// \param bFitToPage           flag to indicate it the image is to be scaled to full page size,
    ///                             defaults to true
    /// \param nResolutionX         horizontal resolution of the image in dpi, defaults to wxivIMAGEPPIX
    /// \param nResolutionY         vertical resolution of the image in dpi, defaults to wxivIMAGEPPIY
    wxIViewPrintout( wxIViewPaintBase *poPaint,
                     const wxString& strTitle= wxT( "wxIViewPrintout" ),
                     bool bFitToPage= true,
                     int nResolutionX= wxivIMAGEPPIX,
                     int nResolutionY= wxivIMAGEPPIY );

    /// \brief From the wxWidgets documentation:
    ///        Called by the framework to obtain information from the application about minimum
    ///        and maximum page values that the user can select, and the required page range to
    ///        be printed. By default this returns 1, 32000 for the page minimum and maximum
    ///        values, and 1, 1 for the required page range. If minPage is zero, the page number
    ///        controls in the print dialog will be disabled.
    ///
    /// \param nMinPage             ...
    /// \param nMaxPage             ...
    /// \param nSelPageFrom         ...
    /// \param nSelPageTo           ...
    virtual void GetPageInfo( int *nMinPage, int *nMaxPage, int *nSelPageFrom, int *nSelPageTo );

    /// \brief From the wxWidgets documentation:
    ///        Should be overridden to return true if the document has this page, or false if not.
    ///        Returning false signifies the end of the document. By default, HasPage behaves as if
    ///        the document has only one page.
    ///
    /// \param nPage                page number to print
    virtual bool HasPage( int nPage );

    /// \brief From the wxWidgets documentation:
    ///        Called by the framework when a page should be printed. Returning false cancels the
    ///        print job. The application can use wxPrintout::GetDC to obtain a device context to
    ///        draw on.
    ///
    /// \param nPage                page number to print
    virtual bool OnPrintPage( int nPage );

  protected:
    wxIViewPaintBase *m_poPaint;    ///< Pointer to the parent object.
    bool m_bFitToPage;              ///< Flag: Should the output scaled to fix the printer page?
    int m_nResolutionX;             ///< Resolution in width direction in DPI.
    int m_nResolutionY;             ///< Resolution in height direction in DPI.
};


/////////////////////////////////////////////////////////////////////////////
//  Class wxIViewPreviewFrame
//
/// \brief A slightly extended print preview window. The only differenct to the
///        base class wxPreviewFrame: It only shows options in its control bar
///        that are usable (e.g. options for turning pages if there are more
///        than one).
///
/// ...
class wxIViewPreviewFrame : public wxPreviewFrame
{
  public:
    /// \brief Standard constructor
    ///
    /// \param poPrintPreview    a pointer to ...
    /// \param poParentWindow    a pointer to the parent window
    /// \param strTitle          a string containing the window title, defaults to "Print Preview"
    /// \param oPos              window position, defaults to wxDefaultPosition
    /// \param oSize             window size, defaults to wxDefaultSize
    /// \param nStyle            window style, defaults to wxDEFAULT_FRAME_STYLE
    /// \param strName           a string containing class information, defaults to "wxIViewPreviewFrame"
    wxIViewPreviewFrame( wxPrintPreview* poPrintPreview,
                         wxWindow* poParentWindow,
                         const wxString& strTitle= wxT( "Print Preview" ),
                         const wxPoint& oPos= wxDefaultPosition,
                         const wxSize& oSize= wxDefaultSize,
                         long  nStyle= wxDEFAULT_FRAME_STYLE,
                         const wxString& strName = wxT( "wxIViewPreviewFrame" ) )
      : wxPreviewFrame( poPrintPreview, poParentWindow, strTitle, oPos, oSize, nStyle,
                        strName )
    {
    }

    /// \brief Create the preview window's control bar.
    ///
    /// Creates a control bar that does not contain buttons for turning pages
    /// forward and backward if there is only one page that can be printed.
    /// The code is inspired on the default implementation of
    /// wxPreviewFrame::CreateControlBar() in ${WXWIN}/src/common.prntbase.cpp .
    virtual void CreateControlBar();
};

#endif // __WXIVIEWPRINTOUT_H

